#version 120

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gnormal;
uniform sampler2D composite;
uniform sampler2D noisetex;

varying vec4 texcoord;
varying vec3 sunlight_color;
varying vec3 ambient_color;
varying float isEyeInWater;

uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float near;
uniform float far;

const float bloomStrength = 0.1;
const float waterFogDensity = 0.9;



float linearizeDepth(float depth) {
    return (2.0 * near) / (far + near - depth * (far - near));
}

vec3 getBloomColor(vec2 texCoords) {
    vec3 bloomColor = vec3(0.0);
    float totalWeight = 0.0;
    
    for (int x = -4; x <= 4; x++) {
        for (int y = -4; y <= 4; y++) {
            vec2 offset = vec2(float(x), float(y)) / vec2(viewWidth, viewHeight);
            float weight = 1.0 / (1.0 + float(x*x + y*y));
            bloomColor += texture2D(gcolor, texCoords + offset).rgb * weight;
            totalWeight += weight;
        }
    }
    
    return bloomColor / totalWeight;
}

void main() {
    vec3 color = texture2D(gcolor, texcoord.st).rgb;
    float depth = texture2D(gdepth, texcoord.st).r;
    
    // Apply underwater fog
    if (isEyeInWater > 0.5) {
        float fogFactor = 1.0 - exp(-waterFogDensity * linearizeDepth(depth));
        vec3 waterColor = vec3(0.1, 0.3, 0.5); // Mostly blue with a hint of white
        color = mix(color, waterColor, fogFactor);
    }
    
    // Apply bloom
    vec3 bloomColor = getBloomColor(texcoord.st);
    color += bloomColor * bloomStrength;

    // Final color adjustment
    color *= sunlight_color;
    color = clamp(color, 0.0, 1.0);
    
    gl_FragColor = vec4(color, 1.0);

}