#version 120


#define composite0
#define GODRAYS 1 // [1 0] 
#include "shaders.settings"

varying vec2 texcoord;
varying vec4 color;
uniform vec3 shadowLightPosition;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D texture;
uniform sampler2D gnormal;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform float viewWidth;
uniform float viewHeight;
uniform float near;
uniform float far;
uniform int isEyeInWater;

vec3 decode (vec2 enc){
    vec2 fenc = enc*4-2;
    float f = dot(fenc,fenc);
    float g = sqrt(1-f/4.0);
    vec3 n;
    n.xy = fenc*g;
    n.z = 1-f/2;
    return n;
}

float cdist(vec2 coord) {
    return clamp(1.0 - max(abs(coord.s-0.5),abs(coord.t-0.5))*2.0, 0.0, 1.0);
}

vec3 screenSpace(vec2 coord, float depth){
    vec4 pos = gbufferProjectionInverse * (vec4(coord, depth, 1.0) * 2.0 - 1.0);
    return pos.xyz/pos.w;
}

vec3 toScreenSpace(vec3 pos) {
    vec4 iProjDiag = vec4(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y, gbufferProjectionInverse[2].zw);
    vec3 p3 = pos * 2.0 - 1.0;
    vec4 fragposition = iProjDiag * p3.xyzz + gbufferProjectionInverse[3];
    return fragposition.xyz / fragposition.w;
}

#if GODRAYS == 1
varying vec2 lightPos;
float land = 1.0-near/far/far;

float getnoise(vec2 pos) {
    return fract(sin(dot(pos ,vec2(18.9898f,28.633f))) * 4378.5453f);
}

vec3 calcRays(vec3 color){
    vec2 deltatexcoord = vec2(lightPos - texcoord) * 0.05; // Reduced from 0.08 to 0.05 for softer rays
    
#if grays_quality == 1
    vec2 noisetc = texcoord;
#elif grays_quality == 2
    vec2 noisetc = texcoord + deltatexcoord*getnoise(texcoord)*1.5; // Reduced noise variation
#endif

    float gr = 1.0;
    const int SAMPLES = 30;
    for (int i = 0; i < SAMPLES; i++) {
        float depth0 = texture2D(depthtex0, noisetc).x;
        noisetc += deltatexcoord;
        float dist = length(noisetc - texcoord);
        float distanceScale = 1.0 - smoothstep(0.0, 1.0, dist);
        gr += dot(step(land, depth0), 1.0) * cdist(noisetc) * (1.0 + distanceScale * 0.3); // Reduced distance contribution
    }
    gr /= float(SAMPLES);

    vec3 gfragpos0 = screenSpace(texcoord.xy, texture2D(depthtex0, texcoord.xy).x);
    float dist = length(gfragpos0);
    float distanceFade = 1.0 - smoothstep(0.0, far * 0.8, dist);
    
    float lightpos = clamp(dot(normalize(gfragpos0.xyz), normalize(shadowLightPosition.xyz)), 0.0, 1.0) 
                     * gr * grays_intensity * 0.7 * (1.0 + distanceFade); // Reduced intensity by 30%
    
    return color *= 1.0 + lightpos * color * (1.0 - isEyeInWater) * (1.0 + distanceFade * 0.3); // Reduced final intensity
}
#endif

void main() {
    vec4 tex = texture2D(texture, texcoord.xy)*color;
    vec3 normal = texture2D(gnormal, texcoord.xy).xyz;
    vec3 newnormal = decode(normal.xy);
    vec3 fragPos = toScreenSpace(vec3(texcoord, texture2D(depthtex0, texcoord.xy).x));

#if GODRAYS == 1
    tex.rgb = calcRays(tex.rgb);
#endif

#ifdef depthbuffer
float c = (2.0 * near) / (far + near - texture2D(depthtex0, texcoord.xy).x * (far - near));
tex.rgb = vec3(c);
#endif

#ifdef draw_refnormals
tex.rgb = normal.rgb;
#endif

    gl_FragData[0] = tex;
    gl_FragData[1] = vec4(0.0);
}
