#version 130


// Simplified and cleaned up

#include "/settings/define.h"

// Varying texture coordinates passed from vertex shader
varying vec2 TexCoords;

// Uniform variables for screen dimensions and depth information
uniform float viewWidth, viewHeight, aspectRatio;
uniform float centerDepthSmooth;

// Projection matrix
uniform mat4 gbufferProjection;

// Color and depth textures
uniform sampler2D colortex0;
uniform sampler2D depthtex1;

// 60 predefined offset values for DoF effect
const vec2 dofOffsets[60] = vec2[60](
    vec2( 0.0 , 0.25 ), vec2(-0.2165, 0.125), vec2(-0.2165, -0.125), vec2( 0, -0.25 ),
    vec2( 0.2165, -0.125), vec2( 0.2165, 0.125), vec2( 0, 0.5 ), vec2(-0.25, 0.433),
    vec2(-0.433, 0.25), vec2(-0.5, 0), vec2(-0.433, -0.25), vec2(-0.25, -0.433),
    vec2( 0, -0.5), vec2( 0.25, -0.433), vec2( 0.433, -0.2), vec2( 0.5, 0),
    vec2( 0.433, 0.25), vec2( 0.25, 0.433), vec2( 0, 0.75), vec2(-0.2565, 0.7048),
    vec2(-0.4821, 0.5745), vec2(-0.51295, 0.375), vec2(-0.7386, 0.1302), vec2(-0.7386, -0.1302),
    vec2(-0.51295, -0.375), vec2(-0.4821, -0.5745), vec2(-0.2565, -0.7048), vec2( 0, -0.75),
    vec2( 0.2565, -0.7048), vec2( 0.4821, -0.5745), vec2( 0.51295, -0.375), vec2( 0.7386, -0.1302),
    vec2( 0.7386, 0.1302), vec2( 0.51295, 0.375), vec2( 0.4821, 0.5745), vec2( 0.2565, 0.7048),
    vec2( 0, 1), vec2(-0.2588, 0.9659), vec2(-0.5, 0.866), vec2(-0.7071, 0.7071),
    vec2(-0.866, 0.5), vec2(-0.9659, 0.2588), vec2(-1, 0), vec2(-0.9659, -0.2588),
    vec2(-0.866, -0.5), vec2(-0.7071, -0.7071), vec2(-0.5, -0.866), vec2(-0.2588, -0.9659),
    vec2( 0, -1), vec2( 0.2588, -0.9659), vec2( 0.5, -0.866), vec2( 0.7071, -0.7071),
    vec2( 0.866, -0.5), vec2( 0.9659, -0.2588), vec2( 1, 0), vec2( 0.9659, 0.2588),
    vec2( 0.866, 0.5), vec2( 0.7071, 0.7071), vec2( 0.5, 0.866), vec2( 0.2588, 0.9659)
);

// Function to calculate Depth of Field effect
vec3 DepthOfField(vec3 color, float depth) {
    vec3 dofResult = vec3(0.0);  // Initialize DoF result
    float isNear = float(depth < 0.56);  // Check if depth is near

    // Compute circle of confusion (CoC) and scale factor based on depth
    float fovScale = gbufferProjection[1][1] / 1.37;
    float coc = max(abs(depth - centerDepthSmooth) * DOF_STRENGTH - 0.01, 0.0);
    coc = coc / sqrt(coc * coc + 0.1);  // Normalize CoC

    // Apply DoF effect if conditions are met
    if (coc > 0.0 && isNear < 0.5) {
        for(int i = 0; i < 60; i++) {
            vec2 offset = dofOffsets[i] * coc * 0.015 * fovScale * vec2(1.0 / aspectRatio, 1.0);
            float lod = log2(viewHeight * aspectRatio * coc * fovScale / 320.0);
            dofResult += texture2DLod(colortex0, TexCoords + offset, lod).rgb;
        }
        dofResult /= 60.0;  // Average the blur samples
    }
    else {
        dofResult = color;  // Return original color if no blur is needed
    }

    return dofResult;
}

void main() {
    // Fetch original color from color texture
    vec3 color = texture2D(colortex0, TexCoords).rgb;

    // Apply Depth of Field if enabled (DOF == 1)
    #if DOF == 1
    float depth = texture2D(depthtex1, TexCoords).r;
    color = DepthOfField(color, depth);
    #endif

    // Output the final color
    gl_FragData[0] = vec4(color, 1.0);
}
