#if !defined(GBUFFERS_BASIC) && !defined(GBUFFERS_SKYBASIC)
    uniform mat4 gbufferModelViewInverse;
    attribute vec3 mc_Entity;
    uniform int isEyeInWater;

    #if defined(GBUFFERS_WATER) || defined(GBUFFERS_HAND_WATER)
        attribute vec4 at_tangent;
        uniform vec3 cameraPosition;
        uniform float frameTimeCounter;
        
        out mat3 TBN;
        out vec4 N;
        
        void setupWaterAttributes() {
            vec3 tangent = normalize(gl_NormalMatrix * at_tangent.xyz);
            vec3 binormal = normalize(gl_NormalMatrix * cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
            vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
            
            TBN = transpose(mat3(tangent, binormal, normal));
            N = vec4(normal, 0.0);
            
            // Set material-specific alpha values
            if(mc_Entity.x == 1) N.a = 0.5;
            if(mc_Entity.x == 2) N.a = 0.7;
        }
    #endif

    out vec3 wPos;
    out vec2 uv0;
    out vec2 uv1;
    varying float v_fogFactor;

    void computeWorldPosition() {
        vec3 viewPos = (gl_ModelViewMatrix * gl_Vertex).xyz;
        wPos = mat3(gbufferModelViewInverse) * viewPos + gbufferModelViewInverse[3].xyz;
        
        // Adjust fog based on whether player is underwater
        float fogStart = 0.0;
        float fogEnd = isEyeInWater == 1 ? 12.0 : 48.0;  // Reduced visibility underwater
        float dist = length(viewPos);
        v_fogFactor = clamp((dist - fogStart) / (fogEnd - fogStart), 0.0, 1.0);
        
        // Increase fog density underwater
        if(isEyeInWater == 1) {
            v_fogFactor = v_fogFactor * 1.5;  // More intense fog underwater
        }
    }

    void setupTextureCoords() {
        uv0 = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
        uv1 = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    }
#endif

out vec4 vColor;

#ifndef GBUFFERS_BASIC
    out vec3 vPos;
#endif

void main() {
    #ifndef GBUFFERS_BASIC
        vPos = (gl_ModelViewMatrix * gl_Vertex).xyz;
    #endif

    vColor = gl_Color;

    #if !defined(GBUFFERS_BASIC) && !defined(GBUFFERS_SKYBASIC)
        setupTextureCoords();
        computeWorldPosition();

        #if defined(GBUFFERS_WATER) || defined(GBUFFERS_HAND_WATER)
            setupWaterAttributes();
        #endif
    #endif

    gl_Position = ftransform();
}