#version 120

varying vec4 color;
varying vec2 texcoord;

uniform sampler2D texture;
uniform float viewHeight;
uniform float viewWidth;
uniform vec3 fogColor;
uniform vec3 skyColor;
uniform float rainStrength;
uniform int worldTime;

void main() {
    vec4 tex = texture2D(texture, texcoord);
    
    // Define winter sky colors
    vec3 winterSkyTop = vec3(0.7, 0.8, 0.9);    // Light blue-white
    vec3 winterSkyHorizon = vec3(0.3647, 0.6353, 0.8863); // Almost white
    
    // Calculate fog density based on vertical position
    float fogDensity = 2.5; // Increased density for winter atmosphere
    float verticalGradient = 1.0 - gl_FragCoord.y / viewHeight;
    float fogFactor = exp(-fogDensity * verticalGradient);
    
    // Add some time-based variation to fog
    float timeScale = float(worldTime) / 24000.0;
    float timeFactor = sin(timeScale * 3.14159 * 2.0) * 0.15 + 0.85; // Reduced variation for stable winter sky
    
    // Blend winter sky colors based on height
    vec3 winterSky = mix(winterSkyHorizon, winterSkyTop, verticalGradient);
    
    // Adjust fog color for winter
    vec3 winterFogColor = vec3(0.9, 0.92, 0.95); // Slightly blue-tinted white fog
    vec3 finalFogColor = mix(winterFogColor, fogColor, rainStrength * 0.3);
    finalFogColor *= timeFactor;
    
    // Blend fog with winter sky
    vec3 baseColor = mix(tex.rgb * color.rgb, winterSky, 0.6);
    vec3 finalColor = mix(finalFogColor, baseColor, fogFactor);
    
    // Add slight blue tint to final output for winter atmosphere
    finalColor = mix(finalColor, finalColor * vec3(0.95, 0.97, 1.0), 0.2);
    
    gl_FragData[0] = vec4(finalColor, 1.0);
}