#version 120

varying vec4 color;
varying vec2 texcoord;
varying float fade;

uniform sampler2D texture;
uniform float viewHeight;
uniform float viewWidth;
uniform vec3 fogColor;
uniform vec3 skyColor;
uniform float rainStrength;
uniform int worldTime;

void main() {
    vec4 tex = texture2D(texture, texcoord);
    
    // Define winter sky colors
    vec3 winterSkyTop = vec3(0.7, 0.8, 0.9);    // Light blue-white
    vec3 winterSkyHorizon = vec3(0.3451, 0.5961, 0.8275); // Almost white
    
    // Calculate fog density based on vertical position
    float fogDensity = 2.5;
    float verticalGradient = 1.0 - gl_FragCoord.y / viewHeight;
    float fogFactor = exp(-fogDensity * verticalGradient);
    
    // Add time-based variation
    float timeScale = float(worldTime) / 24000.0;
    float timeFactor = sin(timeScale * 3.14159 * 2.0) * 0.15 + 0.85;
    
    // Blend winter sky colors
    vec3 winterSky = mix(winterSkyHorizon, winterSkyTop, verticalGradient);
    
    // Adjust fog color for winter
    vec3 winterFogColor = vec3(0.9, 0.92, 0.95);
    vec3 finalFogColor = mix(winterFogColor, fogColor, rainStrength * 0.3);
    finalFogColor *= timeFactor;
    
    // Cloud fade effect
    float cloudOpacity = mix(1.0, 0.3, fade); // Fade clouds from fully opaque to 30% opacity
    vec4 cloudColor = vec4(tex.rgb * color.rgb, tex.a * cloudOpacity);
    
    // Blend clouds with sky
    vec3 baseColor = mix(cloudColor.rgb, winterSky, cloudColor.a < 0.1 ? 1.0 : 0.0);
    vec3 finalColor = mix(finalFogColor, baseColor, fogFactor);
    
    // Add slight blue tint
    finalColor = mix(finalColor, finalColor * vec3(0.95, 0.97, 1.0), 0.2);
    
    gl_FragData[0] = vec4(finalColor, cloudColor.a);
}