#version 120

#define WAVE
#define SHADOWS

attribute vec4 mc_Entity;

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform float frameTimeCounter;
uniform vec3 sunPosition;
uniform vec3 cameraPosition;

varying float isPlant;
varying float isOre;
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 cPos; //Chunked Position
varying vec3 wPos; //World Position
varying vec3 sunPos; //Sun Position
varying vec3 normal;
varying vec4 position;
varying vec4 glcolor;

// Enhanced lighting calculations
vec3 calculateLighting(vec3 normalVector, vec3 sunDir) {
    float diffuse = max(dot(normalVector, sunDir), 0.0);
    return vec3(diffuse);
}

void main() {
    isPlant = 0.0;
    isOre = 0.0;

    gl_Position = ftransform();

    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    glcolor = gl_Color;
    normal = gl_NormalMatrix * gl_Normal;

    // Enhanced normal calculation for better lighting
    vec3 worldNormal = normalize(gl_Normal);
    vec3 lightDir = normalize(sunPosition);
    vec3 lighting = calculateLighting(worldNormal, lightDir);
    glcolor.rgb *= mix(vec3(0.4667, 0.4667, 0.4667), vec3(0.5608, 0.5608, 0.5608), lighting);

    float e = mc_Entity.x;
    if (e == 141.0 || e == 142.0 || e == 59.0 || e == 106.0 || e == 127.0 || e == 175.0 || e == 83.0 || e == 32.0 || e == 37.0 || e == 38.0 || e == 31.0 || e == 115.0 || e == 6.0 || e == 39.0 || e == 40.0 || e == 104.0 || e == 105.0) isPlant = 1.0;
    if (e == 56.0 || e == 73.0 || e == 129.0 || e == 14.0 || e == 15.0 || e == 21.0 || e == 79.0) isOre = 1.0;

    // Additional ore check with enhanced lighting
    if (mc_Entity.x == 11116.0 || mc_Entity.x == 11117.0 || // Diamond Ore
        mc_Entity.x == 11118.0 || mc_Entity.x == 11119.0 || // Emerald Ore
        mc_Entity.x == 11120.0 || mc_Entity.x == 11121.0 || // Gold Ore
        mc_Entity.x == 11122.0 || mc_Entity.x == 11123.0 || // Lapis Ore
        mc_Entity.x == 11124.0 || mc_Entity.x == 11125.0 || // Redstone Ore Unlit
        mc_Entity.x == 11128.0 || mc_Entity.x == 11129.0 || // Redstone Ore Lit
        mc_Entity.x == 11132.0 || mc_Entity.x == 11133.0 || // Iron Ore
        mc_Entity.x == 11136.0 || mc_Entity.x == 11137.0) { // Copper Ore
        isOre = 1.0;
        // Add extra lighting for ores
        glcolor.rgb *= 1.2;
    }

    sunPos = (vec4(sunPosition, 1.0) * gl_ModelViewMatrix).xyz;
    vec4 pos = gbufferModelViewInverse * gl_ModelViewMatrix * gl_Vertex;
    cPos = pos.xyz + cameraPosition;

    #ifdef WAVE
    float wind = cos(frameTimeCounter * 0.3 + cPos.x * 0.02 + cPos.y * 0.04 + cPos.z * 0.002);
    if (isPlant > 0.5) pos.xz += cos(frameTimeCounter * 4.0 + dot(cPos, vec3(3.2, 1.2, 1.6))) * 0.02 * wind * lmcoord.y; //Grasses, flowers, and other
    if (e == 18.0 || e == 161.0) pos.xz += cos(frameTimeCounter * 4.0 + dot(cPos, vec3(1.2, 0.0, 0.6))) * 0.02 * wind * lmcoord.y; //Leaves
    #endif

    wPos = pos.xyz;
    position = gl_ModelViewMatrix * gl_Vertex;

    gl_Position = gl_ProjectionMatrix * gbufferModelView * pos;
}
